@@ -1,4 +1,4 @@
-/* Copyright (C) 2001, 2002, 2003, 2004 by Hans Reiser, licensing governed by
+/* Copyright (C) 2001-2005 by Hans Reiser, licensing governed by
    reiser4progs/COPYING.
    
    trunc.c -- a demo program that truncates the file to the given size. */
@@ -28,7 +28,7 @@ errno_t trunc_cmd(busy_ctx_t *ctx) {
 		return -EIO;
 	}
 	
-	if (objplug(object)->id.group != REG_OBJECT) {
+	if (reiser4_psobj(object)->p.id.group != REG_OBJECT) {
 		aal_error("The object %s is not a regular file.", 
 			  ctx->in.path);
 		goto error_close_object;
