@@ -1,4 +1,4 @@
-/* Copyright 2001, 2002, 2003, 2004 by Hans Reiser, licensing governed by
+/* Copyright 2001-2005 by Hans Reiser, licensing governed by
    reiser4progs/COPYING.
    
    repair/plugin.h - reiser4 plugins repair code methods. */
@@ -22,20 +22,21 @@ typedef enum repair_mode {
 } repair_mode_t;
 
 /* Fixable errors were detected. */
-#define RE_FIXABLE	((uint64_t)1 << 32)
+#define RE_FIXABLE	((int64_t)1 << 32)
 /* Fatal errors were detected. */
-#define RE_FATAL	((uint64_t)1 << 33)
+#define RE_FATAL	((int64_t)1 << 33)
 /* For expansibility. */
-#define RE_LAST		((uint64_t)1 << 34)
+#define RE_LAST		((int64_t)1 << 34)
 
-#define repair_error_fatal(result)   ((result & RE_FATAL) || (result < 0))
+#define repair_error_fatal(result)   ((result < 0) || (result & RE_FATAL))
 
 #define EXCEPTION_TYPE_FSCK EXCEPTION_TYPE_LAST
 
-#define fsck_mess(msg, list...)				\
-	aal_exception_throw(EXCEPTION_TYPE_FSCK,	\
-			    EXCEPTION_OPT_OK,		\
-			    "FSCK: "msg,		\
+#define fsck_mess(msg, list...)					\
+	aal_exception_throw(EXCEPTION_TYPE_FSCK,		\
+			    EXCEPTION_OPT_OK,			\
+			    "FSCK: %s: %d: %s: "msg,		\
+			    __FILE__, __LINE__, __FUNCTION__,	\
 			    ##list)
 
 #define MASTER_PACK_SIGN	"MSTR"
