@@ -1,4 +1,4 @@
-/* Copyright (C) 2001, 2002, 2003, 2004 by Hans Reiser, licensing governed by
+/* Copyright (C) 2001-2005 by Hans Reiser, licensing governed by
    reiser4progs/COPYING.
    
    misc.c -- some common tools for all reiser4 utilities. */
@@ -282,7 +282,10 @@ void misc_upper_case(char *dst, const char *src) {
 }
 
 static errno_t cb_print_plug(reiser4_plug_t *plug, void *data) {
-	printf("\"%s\": %s\n", plug->label, plug->desc);
+	uint8_t w1;
+	w1 = 18 - aal_strlen(plug->label);
+	printf("\"%s\"%*s(id:0x%x type:0x%x) [%s]\n", plug->label, 
+	       w1, " ", plug->id.id, plug->id.type, plug->desc);
 	return 0;
 }
 
