@@ -1,4 +1,4 @@
-/* Copyright (C) 2001, 2002, 2003, 2004 by Hans Reiser, licensing governed by
+/* Copyright (C) 2001-2005 by Hans Reiser, licensing governed by
    reiser4progs/COPYING.
    
    item.c -- common reiser4 item functions. */
@@ -11,9 +11,8 @@ uint32_t reiser4_item_units(reiser4_place_t *place) {
 	aal_assert("umka-1030", place != NULL);
 	aal_assert("umka-1448", place->plug != NULL);
 
-	if (place->plug->o.item_ops->balance->units) {
-		return plug_call(place->plug->o.item_ops->balance,
-				 units, place);
+	if (place->plug->balance->units) {
+		return objcall(place, balance->units);
 	}
 	
 	return 1;
@@ -24,7 +23,7 @@ bool_t reiser4_item_statdata(reiser4_place_t *place) {
 	aal_assert("umka-1831", place != NULL);
 	aal_assert("umka-2382", place->plug != NULL);
 
-	return place->plug->id.group == STAT_ITEM;
+	return place->plug->p.id.group == STAT_ITEM;
 }
 
 /* Returns item type from its plugin */
@@ -32,11 +31,11 @@ rid_t reiser4_item_type(reiser4_place_t *place) {
 	aal_assert("vpf-424", place != NULL);
 	aal_assert("vpf-425", place->plug != NULL);
 	
-	if (place->plug->id.type != ITEM_PLUG_TYPE)
+	if (place->plug->p.id.type != ITEM_PLUG_TYPE)
 		return LAST_ITEM;
 		
-	return (place->plug->id.group < LAST_ITEM ?
-		place->plug->id.group : LAST_ITEM);
+	return (place->plug->p.id.group < LAST_ITEM ?
+		place->plug->p.id.group : LAST_ITEM);
 }
 
 int reiser4_item_mergeable(reiser4_place_t *place1, reiser4_place_t *place2) {
@@ -48,15 +47,15 @@ int reiser4_item_mergeable(reiser4_place_t *place1, reiser4_place_t *place2) {
 		return 0;
 
 	/* Check if mergeable is implemented and calling it if it is. */
-	return place1->plug->o.item_ops->balance->mergeable &&
-		place1->plug->o.item_ops->balance->mergeable(place1, place2);
+	return place1->plug->balance->mergeable &&
+		place1->plug->balance->mergeable(place1, place2);
 }
 #endif
 
 /* Returns 1 if @place points to an nodeptr item. */
-bool_t reiser4_item_branch(reiser4_plug_t *plug) {
+bool_t reiser4_item_branch(reiser4_item_plug_t *plug) {
 	aal_assert("umka-1829", plug != NULL);
-	return (plug->o.item_ops->tree->down_link != NULL);
+	return (plug->p.id.group == PTR_ITEM);
 }
 
 /* Returns maximal possible key may exist in item at @place. */
@@ -69,11 +68,10 @@ errno_t reiser4_item_maxposs_key(reiser4_place_t *place,
 	
 	aal_memcpy(key, &place->key, sizeof(*key));
 
-	if (!place->plug->o.item_ops->balance->maxposs_key)
+	if (!place->plug->balance->maxposs_key)
 		return 0;
 	
-	return plug_call(place->plug->o.item_ops->balance,
-			 maxposs_key, place, key);
+	return objcall(place, balance->maxposs_key, key);
 }
 
 #ifndef ENABLE_MINIMAL
@@ -87,11 +85,10 @@ errno_t reiser4_item_maxreal_key(reiser4_place_t *place,
 
 	aal_memcpy(key, &place->key, sizeof(*key));
 
-	if (!place->plug->o.item_ops->balance->maxreal_key)
+	if (!place->plug->balance->maxreal_key)
 		return 0;
 
-	return plug_call(place->plug->o.item_ops->balance,
-			 maxreal_key, place, key);
+	return objcall(place, balance->maxreal_key, key);
 }
 
 errno_t reiser4_item_update_key(reiser4_place_t *place,
@@ -116,15 +113,13 @@ errno_t reiser4_item_get_key(reiser4_place_t *place,
 
 	/* Getting key from item or node and updating it in passed @place. */
 	key->plug = place->key.plug;
-
-	if (place->plug->o.item_ops->balance->fetch_key &&
+	
+	if (place->plug->balance->fetch_key &&
 	    (place->pos.unit && place->pos.unit != MAX_UINT32))
 	{
-		plug_call(place->plug->o.item_ops->balance,
-			  fetch_key, place, key);
+		objcall(place, balance->fetch_key, key);
 	} else {
-		plug_call(place->node->plug->o.node_ops,
-			  get_key, place->node, &place->pos, key);
+		objcall(place->node, get_key, &place->pos, key);
 	}
 
 	return 0;
@@ -134,20 +129,31 @@ errno_t reiser4_item_get_key(reiser4_place_t *place,
 errno_t reiser4_item_update_link(reiser4_place_t *place,
 				 blk_t blk)
 {
+	trans_hint_t hint;
+	ptr_hint_t ptr;
+	
 	aal_assert("umka-2668", place != NULL);
 	
-	return plug_call(place->plug->o.item_ops->tree,
-			 update_link, place, blk);
+	/* Prepare @hint. */
+	hint.count = 1;
+	hint.specific = &ptr;
+	ptr.start = blk;
+	ptr.width = 1;
+	
+	if (objcall(place, object->update_units, &hint) != 1)
+		return -EIO;
+	
+	return 0;
 }
 
-uint16_t reiser4_item_overhead(reiser4_plug_t *plug) {
+uint16_t reiser4_item_overhead(reiser4_item_plug_t *plug) {
 	aal_assert("vpf-1514", plug != NULL);
-	aal_assert("vpf-1515", plug->id.type == ITEM_PLUG_TYPE);
-
-	if (!plug->o.item_ops->object->overhead)
+	aal_assert("vpf-1515", plug->p.id.type == ITEM_PLUG_TYPE);
+	
+	if (!plug->balance->overhead)
 		return 0;
 	
-	return plug_call(plug->o.item_ops->object, overhead);
+	return plugcall(plug->balance, overhead);
 }
 
 void reiser4_item_set_flag(reiser4_place_t *place, uint16_t flag) {
@@ -157,14 +163,12 @@ void reiser4_item_set_flag(reiser4_place_t *place, uint16_t flag) {
 	aal_assert("vpf-1111", place->node != NULL);
 	aal_assert("vpf-1530", flag < sizeof(flag) * 8 - 1);
 	
-	old = plug_call(place->node->plug->o.node_ops, 
-			get_flags, place->node, place->pos.item);
+	old = objcall(place->node, get_flags, place->pos.item);
 	
 	if (old & (1 << flag))
 		return;
 	
-	plug_call(place->node->plug->o.node_ops, set_flags,
-		  place->node, place->pos.item, old | (1 << flag));
+	objcall(place->node, set_flags, place->pos.item, old | (1 << flag));
 }
 
 void reiser4_item_clear_flag(reiser4_place_t *place, uint16_t flag) {
@@ -174,14 +178,12 @@ void reiser4_item_clear_flag(reiser4_place_t *place, uint16_t flag) {
 	aal_assert("vpf-1532", place->node != NULL);
 	aal_assert("vpf-1533", flag < sizeof(flag) * 8 - 1);
 	
-	old = plug_call(place->node->plug->o.node_ops, 
-			get_flags, place->node, place->pos.item);
+	old = objcall(place->node, get_flags, place->pos.item);
 	
 	if (~old & (1 << flag)) 
 		return;
 	
-	plug_call(place->node->plug->o.node_ops, set_flags, 
-		  place->node, place->pos.item, old & ~(1 << flag));
+	objcall(place->node, set_flags, place->pos.item, old & ~(1 << flag));
 }
 
 void reiser4_item_clear_flags(reiser4_place_t *place) {
@@ -190,13 +192,11 @@ void reiser4_item_clear_flags(reiser4_place_t *place) {
 	aal_assert("vpf-1042", place != NULL);
 	aal_assert("vpf-1112", place->node != NULL);
 	
-	old = plug_call(place->node->plug->o.node_ops, 
-			get_flags, place->node, place->pos.item);
+	old = objcall(place->node, get_flags, place->pos.item);
 	
 	if (!old) return;
 	
-	plug_call(place->node->plug->o.node_ops, set_flags, 
-		  place->node, place->pos.item, 0);
+	objcall(place->node, set_flags, place->pos.item, 0);
 }
 
 bool_t reiser4_item_test_flag(reiser4_place_t *place, uint16_t flag) {
@@ -206,8 +206,7 @@ bool_t reiser4_item_test_flag(reiser4_place_t *place, uint16_t flag) {
 	aal_assert("vpf-1113", place->node != NULL);
 	aal_assert("vpf-1534", flag < sizeof(flag) * 8 - 1);
 	
-	old = plug_call(place->node->plug->o.node_ops, 
-			get_flags, place->node, place->pos.item);
+	old = objcall(place->node, get_flags, place->pos.item);
 	
 	return old & (1 << flag);
 }
@@ -215,31 +214,39 @@ bool_t reiser4_item_test_flag(reiser4_place_t *place, uint16_t flag) {
 void reiser4_item_dup_flags(reiser4_place_t *place, uint16_t flags) {
 	aal_assert("vpf-1540", place != NULL);
 
-	plug_call(place->node->plug->o.node_ops, set_flags, 
-		  place->node, place->pos.item, flags);
+	objcall(place->node, set_flags, place->pos.item, flags);
 }
 
 uint16_t reiser4_item_get_flags(reiser4_place_t *place) {
 	aal_assert("vpf-1541", place != NULL);
 
-	return plug_call(place->node->plug->o.node_ops, get_flags, 
-			 place->node, place->pos.item);
+	return objcall(place->node, get_flags, place->pos.item);
 }
 
 lookup_t reiser4_item_collision(reiser4_place_t *place, coll_hint_t *hint) {
 	aal_assert("vpf-1550", place != NULL);
 	aal_assert("vpf-1551", place->plug != NULL);
 	
-	if (!place->plug->o.item_ops->balance->collision)
+	if (!place->plug->balance->collision)
 		return PRESENT;
 
-	return place->plug->o.item_ops->balance->collision(place, hint);
+	return place->plug->balance->collision(place, hint);
 }
 #endif
 
 /* Return block number nodeptr item contains. */
 blk_t reiser4_item_down_link(reiser4_place_t *place) {
+	trans_hint_t hint;
+	ptr_hint_t ptr;
+	
 	aal_assert("umka-2666", place != NULL);
 	
-	return plug_call(place->plug->o.item_ops->tree, down_link, place);
+	/* Prepare @hint. */
+	hint.count = 1;
+	hint.specific = &ptr;
+
+	if (objcall(place, object->fetch_units, &hint) != 1)
+		return MAX_UINT64;
+	
+	return ptr.start;
 }
