@@ -1,4 +1,4 @@
-/* Copyright (C) 2001, 2002, 2003, 2004 by Hans Reiser, licensing governed by
+/* Copyright (C) 2001-2005 by Hans Reiser, licensing governed by
    reiser4progs/COPYING.
    
    tree.c -- reiser4 tree code. */
@@ -82,8 +82,7 @@ blk_t reiser4_tree_get_root(reiser4_tree_t *tree) {
 	aal_assert("umka-2414", tree->fs != NULL);
 	aal_assert("umka-2415", tree->fs->format != NULL);
 
-	return plug_call(tree->fs->format->ent->plug->o.format_ops, 
-			 get_root, tree->fs->format->ent);
+	return reiser4call(tree->fs->format, get_root);
 }
 
 #ifndef ENABLE_MINIMAL
@@ -93,8 +92,7 @@ uint8_t reiser4_tree_get_height(reiser4_tree_t *tree) {
 	aal_assert("umka-2418", tree->fs != NULL);
 	aal_assert("umka-2419", tree->fs->format != NULL);
 
-	return plug_call(tree->fs->format->ent->plug->o.format_ops, 
-			 get_height, tree->fs->format->ent);
+	return reiser4call(tree->fs->format, get_height);
 }
 
 /* As @node already lies in @tree->nodes hash table and it is going to change
@@ -303,6 +301,7 @@ errno_t reiser4_tree_load_root(reiser4_tree_t *tree) {
 }
 
 #if 0
+/* DEBUGGING */
 static errno_t cb_count_children(reiser4_place_t *place, void *data) {
 	uint32_t *count = (uint32_t *)data;
 	blk_t blk;
@@ -331,7 +330,7 @@ static errno_t debug_node_check_keys(reiser4_node_t *node) {
 	count = reiser4_node_items(node);
 
 	for (pos.item = 0; pos.item < count; pos.item++) {
-		plug_call(node->plug->o.node_ops, 
+		plug_call(node->plug->pl.node, 
 			  get_key, node, &pos, &key);
 
 		if (pos.item && reiser4_key_compfull(&prev, &key) > 0)
@@ -377,12 +376,32 @@ errno_t reiser4_tree_assign_root(reiser4_tree_t *tree,
 }
 #endif
 
+errno_t reiser4_tree_mpressure(reiser4_tree_t *tree) {
+	errno_t res;
+	
+	/* Check for memory pressure event. If memory pressure is uppon us, we
+	   call memory cleaning function. For now we call tree_adjust() in order
+	   to release not locked nodes. */
+	if (!tree->mpc_func || !tree->mpc_func(tree))
+		return 0;
+
+	/* Adjusting the tree as memory pressure is here. */
+	if ((res = reiser4_tree_adjust(tree))) {
+		aal_error("Can't adjust tree.");
+		return res;
+	}
+
+	return 0;
+}
+
 /* Registers passed node in tree and connects left and right neighbour
    nodes. This function does not do any tree modifications. */
 errno_t reiser4_tree_connect_node(reiser4_tree_t *tree,
 				  reiser4_node_t *parent, 
 				  reiser4_node_t *node)
 {
+	errno_t res;
+	
 	aal_assert("umka-1857", tree != NULL);
 	aal_assert("umka-2261", node != NULL);
 
@@ -403,29 +422,19 @@ errno_t reiser4_tree_connect_node(reiser4_tree_t *tree,
 	if (reiser4_tree_hash_node(tree, node))
 		return -EINVAL;
 
-	/* Check for memory pressure event. If memory pressure is uppon us, we
-	   call memory cleaning function. For now we call tree_adjust() in order
-	   to release not locked nodes. */
-	if (tree->mpc_func && tree->mpc_func(tree)) {
-		/* Adjusting the tree as memory pressure is here. */
-		reiser4_node_lock(node);
+	reiser4_node_lock(node);
+	if ((res = reiser4_tree_mpressure(tree))) {
+		aal_error("Can't connect node %llu.",
+			  node->block->nr);
+
+		if (parent)
+			reiser4_node_unlock(parent);
 		
-		if (reiser4_tree_adjust(tree)) {
-			aal_error("Can't adjust tree during connect "
-				  "node %llu.", node->block->nr);
-			
-			reiser4_node_unlock(node);
-			if (parent)
-				reiser4_node_unlock(parent);
-			
-			reiser4_tree_unhash_node(tree, node);
-			return -EINVAL;
-		}
-		
-		reiser4_node_unlock(node);
+		reiser4_tree_unhash_node(tree, node);
 	}
+	reiser4_node_unlock(node);
 	
-	return 0;
+	return res;
 }
 
 /* Remove specified child from the node children list. Updates all neighbour
@@ -471,7 +480,8 @@ errno_t reiser4_tree_disconnect_node(reiser4_tree_t *tree,
 #ifndef ENABLE_MINIMAL
 /* Updates all internal node loaded children positions in parent. */
 static errno_t reiser4_tree_update_node(reiser4_tree_t *tree,
-					reiser4_node_t *node)
+					reiser4_node_t *node,
+					uint8_t start, uint8_t end)
 {
 	uint32_t i;
 	errno_t res;
@@ -480,7 +490,7 @@ static errno_t reiser4_tree_update_node(reiser4_tree_t *tree,
 	aal_assert("umka-3036", node != NULL);
 	aal_assert("umka-3037", reiser4_node_items(node) > 0);
 	
-	for (i = 0; i < reiser4_node_items(node); i++) {
+	for (i = start; i < end; i++) {
 		blk_t blk;
 		uint32_t j;
 
@@ -614,7 +624,7 @@ reiser4_node_t *reiser4_tree_child_node(reiser4_tree_t *tree,
 
 	blk = reiser4_item_down_link(place);
 	if (!(node = reiser4_tree_load_node(tree, place->node, blk))) {
-		aal_error("Can't load root node %llu.", blk);
+		aal_error("Can't load child node %llu.", blk);
 		return NULL;
 	}
 
@@ -755,7 +765,7 @@ errno_t reiser4_tree_next_place(reiser4_tree_t *tree,
 
 	aal_memcpy(next, place, sizeof(*place));
 	next->pos.item++;
-	next->pos.unit = MAX_UINT32;
+	next->pos.unit = 0;
 	
 	if (reiser4_tree_adjust_place(tree, next, next))
 		return -EINVAL;
@@ -768,8 +778,17 @@ errno_t reiser4_tree_next_place(reiser4_tree_t *tree,
 	
 	/* If nodeptr item go down. */
 	while (reiser4_item_branch(next->plug)) {
-		if (!(next->node = reiser4_tree_child_node(tree, next)))
+		blk_t blk;
+		
+		blk = reiser4_item_down_link(next);
+		
+		next->node = reiser4_tree_load_node(tree, next->node, blk);
+		
+		if (!next->node) {
+			aal_error("Can't load a child node %llu of the node"
+				  " (%llu).", blk, node->block->nr);
 			goto error;
+		}
 
 		if (reiser4_place_first(next))
 			goto error;
@@ -811,16 +830,14 @@ errno_t reiser4_tree_place_key(reiser4_tree_t *tree,
 	if (reiser4_place_fetch(&next))
 		return -EINVAL;
 	
-	reiser4_item_get_key(&next, key);
-
-	return 0;
+	return reiser4_item_get_key(&next, key);
 }
 
 #ifndef ENABLE_MINIMAL
 /* Gets the key of the next item. */
-static errno_t reiser4_tree_next_key(reiser4_tree_t *tree, 
-				     reiser4_place_t *place, 
-				     reiser4_key_t *key) 
+errno_t reiser4_tree_next_key(reiser4_tree_t *tree, 
+			      reiser4_place_t *place, 
+			      reiser4_key_t *key) 
 {
 	reiser4_place_t temp;
 	
@@ -839,12 +856,12 @@ static errno_t reiser4_tree_next_key(reiser4_tree_t *tree,
 reiser4_node_t *reiser4_tree_alloc_node(reiser4_tree_t *tree,
 					uint8_t level)
 {
+	reiser4_node_plug_t *plug;
 	reiser4_node_t *node;
 	uint32_t stamp;
 	errno_t res;
 	blk_t blk;
 	
-	
 	reiser4_format_t *format;
     
 	aal_assert("umka-756", tree != NULL);
@@ -856,11 +873,11 @@ reiser4_node_t *reiser4_tree_alloc_node(reiser4_tree_t *tree,
 	/* Setting up of the free blocks in format. */
 	if ((res = reiser4_format_dec_free(format, 1)))
 		return NULL;
-
+	
+	plug = (reiser4_node_plug_t *)tree->ent.tset[TSET_NODE];
+	
 	/* Creating new node. */
-	if (!(node = reiser4_node_create(tree, tree->ent.tpset[TPSET_NODE],
-					 blk, level))) 
-	{
+	if (!(node = reiser4_node_create(tree, plug, blk, level))) {
 		aal_error("Can't initialize new fake node.");
 		return NULL;
 	}
@@ -956,8 +973,8 @@ static int cb_blocks_comp_func(void *key1, void *key2,
 inline uint32_t reiser4_tree_target_level(reiser4_tree_t *tree,
 					  reiser4_plug_t *plug)
 {
-	return (plug->id.group == TAIL_ITEM) ?
-		LEAF_LEVEL : TWIG_LEVEL;
+	return (plug->id.group == EXTENT_ITEM) ?
+		TWIG_LEVEL : LEAF_LEVEL;
 }
 
 #endif
@@ -994,7 +1011,7 @@ errno_t reiser4_tree_root_key(reiser4_tree_t *tree,
 	aal_assert("umka-1949", tree != NULL);
 	aal_assert("umka-1950", key != NULL);
 
-	key->plug = tree->ent.tpset[TPSET_KEY];
+	key->plug = (reiser4_key_plug_t *)tree->ent.tset[TSET_KEY];
 	
 #ifndef ENABLE_MINIMAL
 	locality = reiser4_oid_root_locality(tree->fs->oid);
@@ -1003,8 +1020,8 @@ errno_t reiser4_tree_root_key(reiser4_tree_t *tree,
 	locality = REISER4_ROOT_LOCALITY;
 	objectid = REISER4_ROOT_OBJECTID;
 #endif
-	return plug_call(key->plug->o.key_ops, build_generic, key,
-			 KEY_STATDATA_TYPE, locality, 0, objectid, 0);
+	return objcall(key, build_generic, KEY_STATDATA_TYPE, 
+		       locality, 0, objectid, 0);
 }
 
 #ifndef ENABLE_MINIMAL
@@ -1052,8 +1069,8 @@ reiser4_tree_t *reiser4_tree_init(reiser4_fs_t *fs) {
 	}
 
 #endif
-	/* Initializing the tpset. */
-	if (reiser4_pset_init(tree))
+	/* Initializing the tset. */
+	if (reiser4_tset_init(tree))
 		goto error_free_data;
 
 	/* Building tree root key. It is used in tree lookup, etc. */
@@ -1067,7 +1084,7 @@ reiser4_tree_t *reiser4_tree_init(reiser4_fs_t *fs) {
  error_free_data:
 #ifndef ENABLE_MINIMAL
 	aal_hash_table_free(tree->blocks);
-error_free_nodes:
+ error_free_nodes:
 #endif
 	aal_hash_table_free(tree->nodes);
  error_free_tree:
@@ -1155,19 +1172,15 @@ static errno_t reiser4_tree_alloc_extent(reiser4_tree_t *tree,
 		int first_time = 1;
 
 
-		if (plug_call(place->plug->o.item_ops->object,
-			      fetch_units, place, &hint) != 1)
-		{
+		if (objcall(place, object->fetch_units, &hint) != 1)
 			return -EIO;
-		}
 
 		/* Check if we have accessed unallocated extent. */
 		if (ptr.start != EXTENT_UNALLOC_UNIT)
 			continue;
 
 		/* Getting unit key. */
-		plug_call(place->plug->o.item_ops->balance,
-			  fetch_key, place, &key);
+		objcall(place, balance->fetch_key, &key);
 
 		/* Loop until all units get allocated. */
 		for (blocks = 0, width = ptr.width; width > 0; width -= ptr.width) {
@@ -1186,8 +1199,8 @@ static errno_t reiser4_tree_alloc_extent(reiser4_tree_t *tree,
 				flags = reiser4_item_get_flags(place);
 
 				/* Updating extent unit at @place->pos.unit. */
-				if (plug_call(place->plug->o.item_ops->object,
-					      update_units, place, &hint) != 1)
+				if (objcall(place, object->update_units, 
+					    &hint) != 1)
 				{
 					return -EIO;
 				}
@@ -1241,11 +1254,9 @@ static errno_t reiser4_tree_alloc_extent(reiser4_tree_t *tree,
 				aal_hash_table_remove(tree->blocks, &key);
 
 				/* Updating the key to find next data block */
-				offset = plug_call(key.plug->o.key_ops,
-						   get_offset, &key);
+				offset = objcall(&key, get_offset);
 
-				plug_call(key.plug->o.key_ops, set_offset,
-					  &key, offset + blksize);
+				objcall(&key, set_offset, offset + blksize);
 			}
 			
 			blocks += ptr.width;
@@ -1273,16 +1284,13 @@ static errno_t cb_node_adjust(reiser4_tree_t *tree, reiser4_node_t *node) {
 		if (reiser4_tree_root_node(tree, node))
 			reiser4_tree_set_root(tree, blk);
 		
-		if (node->p.node && reiser4_item_update_link(&node->p, blk))
-			return -EIO;
-
-		/* Rehashing node in @tree->nodes hash table. */
-		reiser4_tree_rehash_node(tree, node, blk);
-		
-		if (!reiser4_tree_root_node(tree, node)) {
-			if ((res = reiser4_node_update_ptr(node)))
+		if (node->p.node) {
+			if ((res = reiser4_item_update_link(&node->p, blk)))
 				return res;
 		}
+		
+		/* Rehashing node in @tree->nodes hash table. */
+		reiser4_tree_rehash_node(tree, node, blk);
 	}
 
 	return 0;
@@ -1295,7 +1303,7 @@ static errno_t cb_nodeptr_adjust(reiser4_tree_t *tree, reiser4_place_t *place) {
 	   we have to do so, considering, that this is up to tree to know about
 	   items type in it. Probably this is why tree should be plugin too to
 	   handle things like this in more flexible manner. */
-	if (place->plug->id.group != EXTENT_ITEM) 
+	if (place->plug->p.id.group != EXTENT_ITEM) 
 		return 0;
 	
 	/* Allocating unallocated extent item at @place. */
@@ -1611,7 +1619,7 @@ lookup_t reiser4_tree_collision(reiser4_tree_t *tree,
 		return PRESENT;
 	
 	/* If type does not match, there is no collision found. */
-	if (place->plug->id.group != hint->type)
+	if (place->plug->p.id.group != hint->type)
 		return PRESENT;
 
 	/* Key collisions handling. Sequentional search by name. */	
@@ -1664,7 +1672,7 @@ static errno_t reiser4_tree_collision_start(reiser4_tree_t *tree,
                                                                                      
         if (reiser4_place_fetch(place))
                 return -EINVAL;
-                                                                                     
+	
         aal_memcpy(&walk, place, sizeof(*place));
                                                                                      
         /* Main loop until leftmost node reached. */
@@ -1687,7 +1695,7 @@ static errno_t reiser4_tree_collision_start(reiser4_tree_t *tree,
 			   get out here. This clause is needed for the case 
 			   of corruption when not directory item has a NAME 
 			   minor. */
-			if (walk.plug->o.item_ops->balance->maxposs_key) {
+			if (walk.plug->balance->maxposs_key) {
 				reiser4_key_t maxkey;
 
 				reiser4_item_maxposs_key(&walk, &maxkey);
@@ -1698,13 +1706,13 @@ static errno_t reiser4_tree_collision_start(reiser4_tree_t *tree,
 			
                         /* If item's lookup is implemented, we use it. Item key
                            comparing is used otherwise. */
-                        if (walk.plug->o.item_ops->balance->lookup) {
+                        if (walk.plug->balance->lookup) {
 				lookup_hint_t lhint;
 
 				lhint.key = key;
 				
-                                switch (plug_call(walk.plug->o.item_ops->balance,
-                                                  lookup, &walk, &lhint, FIND_EXACT))
+                                switch (objcall(&walk, balance->lookup, 
+						&lhint, FIND_EXACT))
                                 {
                                 case PRESENT:
                                         aal_memcpy(place, &walk, sizeof(*place));
@@ -1789,6 +1797,7 @@ lookup_t reiser4_tree_lookup(reiser4_tree_t *tree, lookup_hint_t *hint,
 	while (1) {
 		uint32_t clevel;
 		lookup_bias_t cbias;
+		blk_t blk;
 
 		clevel = reiser4_node_get_level(place->node);
 		cbias = (clevel > hint->level ? FIND_EXACT : bias);
@@ -1838,15 +1847,20 @@ lookup_t reiser4_tree_lookup(reiser4_tree_t *tree, lookup_hint_t *hint,
 		
 		if (reiser4_place_fetch(place))
 			restore_and_exit(-EIO);
-
+		
 		/* Checking is item at @place is nodeptr one. If not, we correct
 		   posision back. */
 		if (!reiser4_item_branch(place->plug))
 			restore_and_exit(res);
-
+		
 		/* Loading node by its nodeptr item at @place. */
-		if (!(place->node = reiser4_tree_child_node(tree, place)))
+		blk = reiser4_item_down_link(place);
+		if (!(place->node = reiser4_tree_load_node(tree, place->node,
+							   blk)))
+		{
+			aal_error("Can't load child node %llu.", blk);
 			restore_and_exit(-EIO);
+		}
 
 		/* Zero the plug pointer. */
 		place->plug = NULL;
@@ -1948,7 +1962,7 @@ errno_t reiser4_tree_attach_node(reiser4_tree_t *tree, reiser4_node_t *node,
 	hint.count = 1;
 	hint.specific = &ptr;
 	hint.shift_flags = flags;
-	hint.plug = tree->ent.tpset[TPSET_NODEPTR];
+	hint.plug = (reiser4_item_plug_t *)tree->ent.tset[TSET_NODEPTR];
 
 	ptr.width = 1;
 	ptr.start = node->block->nr;
@@ -2152,10 +2166,12 @@ errno_t reiser4_tree_shift(reiser4_tree_t *tree, reiser4_place_t *place,
 {
 	errno_t res;
 	reiser4_node_t *node;
-	reiser4_node_t *update;
+	reiser4_node_t *right;
 
 	shift_hint_t hint;
 	reiser4_key_t lkey;
+	
+	uint8_t start;
 
 	aal_assert("umka-1225", tree != NULL);
 	aal_assert("umka-1226", place != NULL);
@@ -2169,6 +2185,10 @@ errno_t reiser4_tree_shift(reiser4_tree_t *tree, reiser4_place_t *place,
 	hint.control = flags;
 	hint.pos = place->pos;
 
+	/* Needed for the left shift. */
+	start = reiser4_node_items(neig);
+	start = start ? start - 1 : 0;
+	
 	/* Perform node shift from @node to @neig. */
 	if ((res = reiser4_node_shift(node, neig, &hint)))
 		return res;
@@ -2182,23 +2202,23 @@ errno_t reiser4_tree_shift(reiser4_tree_t *tree, reiser4_place_t *place,
 	if (hint.control & SF_UPDATE_POINT)
 		place->pos = hint.pos;
 
-	update = (hint.control & SF_ALLOW_LEFT) ? node : neig;
+	right = (hint.control & SF_ALLOW_LEFT) ? node : neig;
 
 	/* Check if we need update key in insert part of tree. That is if source
 	   node is not empty and there was actually moved at least one item or
 	   unit. */
-	if (reiser4_node_items(update) > 0 && hint.update) {
+	if (reiser4_node_items(right) > 0 && hint.update) {
 		/* Check if node is connected to tree or it is not root and
 		   updating left delimiting keys if it makes sense at all. */
-		if (update->p.node != NULL) {
+		if (right->p.node != NULL) {
 			reiser4_place_t parent;
 
-			/* Getting leftmost key from @update. */
-			reiser4_node_leftmost_key(update, &lkey);
+			/* Getting leftmost key from @right. */
+			reiser4_node_leftmost_key(right, &lkey);
 
 			/* Recursive updating of all internal keys that supposed
 			   to be updated. */
-			aal_memcpy(&parent, &update->p, sizeof(parent));
+			aal_memcpy(&parent, &right->p, sizeof(parent));
 				
 			if ((res = reiser4_tree_update_keys(tree, &parent, &lkey)))
 				return res;
@@ -2207,14 +2227,22 @@ errno_t reiser4_tree_shift(reiser4_tree_t *tree, reiser4_place_t *place,
 
 	/* Updating @node and @neig children's parent position. */
 	if (reiser4_node_get_level(node) > LEAF_LEVEL) {
-		if (reiser4_node_items(node) > 0) {
-			if ((res = reiser4_tree_update_node(tree, node)))
+		reiser4_node_t *left = (hint.control & SF_ALLOW_LEFT) ? neig : NULL;
+
+		if (left && reiser4_node_items(left) > 0) {
+			if ((res = reiser4_tree_update_node(tree, left, start, 
+							    reiser4_node_items(left))))
+			{
 				return res;
+			}
 		}
 	
-		if (reiser4_node_items(neig) > 0) {
-			if ((res = reiser4_tree_update_node(tree, neig)))
+		if (reiser4_node_items(right) > 0) {
+			if ((res = reiser4_tree_update_node(tree, right, 0, 
+							    reiser4_node_items(right))))
+			{
 				return res;
+			}
 		}
 	}
 	
@@ -2824,10 +2852,13 @@ static inline errno_t tree_post_modify(reiser4_tree_t *tree,
 
 	/* Update @place->node children's parent pointers. */
 	if (reiser4_node_get_level(place->node) > LEAF_LEVEL) {
-		if ((res = reiser4_tree_update_node(tree, place->node)))
+		if ((res = reiser4_tree_update_node(tree, place->node, 
+						    place->pos.item, items)))
+		{
 			return res;
+		}
 	}
-
+	
 	return 0;
 }
 
@@ -2883,12 +2914,16 @@ int64_t reiser4_tree_modify(reiser4_tree_t *tree, reiser4_place_t *place,
 			return -ENOSPC;
 	}
 
+	reiser4_node_lock(place->node);
+	
 	/* Inserting/writing data to node. */
 	if ((write = modify_func(place->node, &place->pos, hint)) < 0) {
 		aal_error("Can't insert data to node %llu.",
 			  place_blknr(place));
 		return write;
 	}
+	
+	reiser4_node_unlock(place->node);
 
 	if ((res = tree_post_modify(tree, place)))
 		return res;
@@ -2944,15 +2979,13 @@ static errno_t cb_tree_prep_insert(reiser4_place_t *place, trans_hint_t *hint) {
 	hint->len = 0;
 	hint->overhead = 0;
 
-	return plug_call(hint->plug->o.item_ops->object,
-			 prep_insert, place, hint);
+	return plugcall(hint->plug->object, prep_insert, place, hint);
 }
 
 static errno_t cb_tree_insert(reiser4_node_t *node, pos_t *pos, 
 			      trans_hint_t *hint) 
 {
-	return plug_call(node->plug->o.node_ops, 
-			 insert, node, pos, hint);
+	return objcall(node, insert, pos, hint);
 }
 
 /* Inserts data to the tree. This function is used for inserting items which are
@@ -2972,15 +3005,13 @@ static errno_t cb_tree_prep_write(reiser4_place_t *place, trans_hint_t *hint) {
 	hint->len = 0;
 	hint->overhead = 0;
 
-	return plug_call(hint->plug->o.item_ops->object,
-			 prep_write, place, hint);
+	return plugcall(hint->plug->object, prep_write, place, hint);
 }
 
 static errno_t cb_tree_write(reiser4_node_t *node, 
 			     pos_t *pos, trans_hint_t *hint) 
 {
-	return plug_call(node->plug->o.node_ops, 
-			 write, node, pos, hint);
+	return objcall(node, write, pos, hint);
 }
 
 int64_t reiser4_tree_write(reiser4_tree_t *tree, reiser4_place_t *place,
@@ -3185,10 +3216,12 @@ errno_t reiser4_tree_scan(reiser4_tree_t *tree,
 		hint.level = LEAF_LEVEL;
 		hint.collision = NULL;
 
-                /* Lookup the key. */
-                if ((lookup = reiser4_tree_lookup(tree, &hint, FIND_EXACT, 
+                /* Lookup the key. FIXME: it is possible to spped it up. */
+                if ((lookup = reiser4_tree_lookup(tree, &hint, FIND_EXACT,
 						  &place)) < 0)
+		{
                         return lookup;
+		}
 
 		pos = &place.pos;
 
