@@ -1,4 +1,4 @@
-/* Copyright (C) 2001, 2002, 2003, 2004 by Hans Reiser, licensing governed by
+/* Copyright (C) 2001-2005 by Hans Reiser, licensing governed by
    reiser4progs/COPYING.
 
    backup.c -- filesystem backup methods. */
@@ -76,7 +76,8 @@ errno_t repair_backup_unpack(reiser4_fs_t *fs, aal_stream_t *stream) {
 }
 
 static int repair_backup_hint_cmp(backup_hint_t *list,
-				  backup_hint_t *hint)
+				  backup_hint_t *hint, 
+				  int size)
 {
 	if (list->block.size < hint->block.size)
 		return -1;
@@ -86,7 +87,7 @@ static int repair_backup_hint_cmp(backup_hint_t *list,
 
 	return aal_memcmp(list->block.data, 
 			  hint->block.data, 
-			  hint->block.size);
+			  size);
 }
 
 static backup_hint_t *repair_backup_hint_create(backup_hint_t *hint,
@@ -162,7 +163,7 @@ static errno_t cb_save_backup(uint64_t blk, uint64_t count, void *data) {
 	aal_list_foreach_forward(blocks, at) {
 		cur = (backup_hint_t *)at->data;
 		
-		cmp = repair_backup_hint_cmp(cur, &backup->hint);
+		cmp = repair_backup_hint_cmp(cur, &backup->hint, cur->block.size);
 
 		/* Such a backup is found already, increment the counter. */
 		if (!cmp)
@@ -229,7 +230,7 @@ extern errno_t reiser4_backup_layout_body(reiser4_alloc_t *alloc,
        ids, block size, etc.
    (3) Make a list of backups that could be correct backups with counters for 
        every backup version.
-   (4) Get the backup version that matches the fs metadata (master sb, etc).
+   (4) Create a fresh backup on the fs metadata (master sb, etc).
    (5) Make a decision what version is the correct as the maximum of function:
        
        f = [ 66 % * MATCHES / COUNT ] + [ 33 % * META ]
@@ -237,7 +238,7 @@ extern errno_t reiser4_backup_layout_body(reiser4_alloc_t *alloc,
        MATCHES - count of matched block contents;
        TOTAL - total block count for this backup type (it depends on allocator 
        plugin, block size);
-       META - 1 if matches the fs metadata, 0 if not;
+       META - 1 if read block matches the fresh backup, 0 if not;
 
    When the most valuable backup block is choosed and master + format are opened,
    but they do not match each other, another decision if the backup or the 
@@ -317,9 +318,9 @@ reiser4_backup_t *repair_backup_open(reiser4_fs_t *fs, uint8_t mode) {
 			}
 			
 			/* Allocate a block alloc entity. */
-			if (!(alloc.ent = plug_call(plug->o.alloc_ops, 
-						    create, fs->device, 
-						    blksize, blocks)))
+			if (!(alloc.ent = plugcall((reiser4_alloc_plug_t *)plug,
+						   create, fs->device, blksize, 
+						   blocks)))
 			{
 				aal_error("Can't create the allocator '%s' on "
 					  "%s.", plug->label, fs->device->name);
@@ -361,7 +362,8 @@ reiser4_backup_t *repair_backup_open(reiser4_fs_t *fs, uint8_t mode) {
 			}
 			
 			/* Close alloc entity. */
-			plug_call(plug->o.alloc_ops, close, alloc.ent);
+			plugcall((reiser4_alloc_plug_t *)plug, 
+				 close, alloc.ent);
 		}
 
 		aal_block_fini(&backup->hint.block);
@@ -417,9 +419,9 @@ reiser4_backup_t *repair_backup_open(reiser4_fs_t *fs, uint8_t mode) {
 			}
 
 			/* Allocathe a block alloc entity. */
-			if (!(alloc.ent = plug_call(plug->o.alloc_ops, create,
-						    fs->device, blksize,
-						    ondisk->blocks)))
+			if (!(alloc.ent = plugcall((reiser4_alloc_plug_t *)plug, 
+						   create, fs->device, blksize,
+						   ondisk->blocks)))
 			{
 				aal_error("Can't create the allocator '%s' on "
 					  "%s.", plug->label, fs->device->name);
@@ -434,7 +436,7 @@ reiser4_backup_t *repair_backup_open(reiser4_fs_t *fs, uint8_t mode) {
 				goto error_free_alloc;
 			}
 
-			plug_call(plug->o.alloc_ops, close, alloc.ent);
+			plugcall((reiser4_alloc_plug_t *)plug, close, alloc.ent);
 		}
 	} 
 	
@@ -449,11 +451,14 @@ reiser4_backup_t *repair_backup_open(reiser4_fs_t *fs, uint8_t mode) {
 		matched = 0;
 		
 		/* Compare with the ondisk backup if needed. */
-		if (ondisk && !found) {
-			matched = (repair_backup_hint_cmp(hint, ondisk) ? 
-				   0 : 1);
+		if (ondisk) {
+			/* Compare only untill the pset, as @ondisk does not 
+			   have the  */
+			matched = repair_backup_hint_cmp(hint, ondisk, 
+					ondisk->off[BK_PSET]) ? 0 : 1;
 		
-			found = matched;
+			if (!found)
+				found = matched;
 		}
 		
 		weight = hint->count * 66 / hint->total + matched * 33;
@@ -463,43 +468,41 @@ reiser4_backup_t *repair_backup_open(reiser4_fs_t *fs, uint8_t mode) {
 			app = hint;
 
 			/* Set @found to 2 if some previous, not the current 
-			   hint, matches @ondisk hint. */
+			   backup, matches @ondisk hint. The chosen backup does
+			   not match the fs metadata. */
 			if (found && !matched)
 				found = 2;
 		}
 	}
 
-	if (mode != RM_BUILD) {
-		/* All backup blocks must be found and must match 
-		   the ondisk one. */
-		if (found != 1) {
-			fsck_mess("Found backup does not match to the on-disk "
-				  "filesystem metadata.");
-			goto error_free_ondisk;
-		}
-		
-		if (app->total != app->count) {
-			fsck_mess("Only %llu of %llu backup blocks found.",
-				  app->count, app->total);
-			goto error_free_ondisk;
-		}
-	} else {
-		/* No backup block is found. */
-		if (!app) goto error_free_ondisk;
-		
-		/* The only 1 backup block is found and it does not match 
-		   the @ondisk data */
-		if (ondisk && found != 1 && app->count == 1) {
-			/* Some more backup blocks must present. */
-			if (app->count != app->total)
-				goto error_free_ondisk;
+	/* If no backup block is found. */
+	if (!app) goto error_free_ondisk;
+	
+	if (app->total != app->count) {
+		fsck_mess("Only %llu of %llu backup blocks are found.",
+			  app->count, app->total);
 
-			/* The only 1 backup block must present according to 
-			   both this backup block and the @ondisk data, take
-			   @ondisk as the correct version. */
-			if (ondisk->total == 1)
-				goto error_free_ondisk;
-		}
+		if (mode != RM_BUILD)
+			goto error_free_ondisk;
+	}
+	
+	/* If only 1 block of the backup is found and it does not match 
+	   the @ondisk data */
+	if (ondisk && found != 1 && app->count == 1) {
+		/* Some more backup blocks must present. */
+		if (app->count != app->total)
+			goto error_free_ondisk;
+		
+		/* The only 1 backup block must present according to 
+		   both this backup block and the @ondisk data, take
+		   @ondisk as the correct version. */
+		//if (ondisk->total == 1)
+		//        goto error_free_ondisk;
+	}
+	
+	if (found != 1) {
+		fsck_mess("Found backup does not match to the on-disk "
+			  "filesystem metadata.");
 	}
 	
 	if (ondisk) {
@@ -527,7 +530,7 @@ reiser4_backup_t *repair_backup_open(reiser4_fs_t *fs, uint8_t mode) {
 	return backup;
 
  error_free_alloc:
-	plug_call(plug->o.alloc_ops, close, alloc.ent);
+	plugcall((reiser4_alloc_plug_t *)plug, close, alloc.ent);
  error_fini_backup:
 	if (backup->hint.block.data) 
 		aal_block_fini(&backup->hint.block);
@@ -548,7 +551,9 @@ reiser4_backup_t *repair_backup_reopen(reiser4_fs_t *fs) {
 	if (!fs->backup)
 		return backup;
 
-	if (!repair_backup_hint_cmp(&backup->hint, &fs->backup->hint)) {
+	if (!repair_backup_hint_cmp(&backup->hint, &fs->backup->hint, 
+				    backup->hint.block.size))
+	{
 		/* Backup matches, nothing has been fixed. */
 		reiser4_backup_close(backup);
 		return fs->backup;
