@@ -1,4 +1,4 @@
-/* Copyright 2001, 2002, 2003, 2004 by Hans Reiser, licensing governed by 
+/* Copyright 2001-2005 by Hans Reiser, licensing governed by 
    reiser4progs/COPYING.
    
    librepair/filesystem.c - methods are needed mostly by fsck for work 
@@ -481,7 +481,7 @@ errno_t repair_fs_lost_key(reiser4_fs_t *fs, reiser4_key_t *key) {
 	aal_assert("vpf-1553", fs != NULL);
 	aal_assert("vpf-1554", key != NULL);
 
-	key->plug = fs->tree->ent.tpset[TPSET_KEY];
+	key->plug = (reiser4_key_plug_t *)fs->tree->ent.tset[TSET_KEY];
 	locality = reiser4_oid_root_objectid(fs->oid);
 	objectid = repair_oid_lost_objectid(fs->oid);
 	
@@ -510,7 +510,10 @@ errno_t repair_fs_check_backup(aal_device_t *device, backup_hint_t *hint) {
 		return res;
 
 	/* Backup the format backup structure. */
-	return repair_format_check_backup(device, hint);
+	if ((res = repair_format_check_backup(device, hint)))
+		return res;
+	
+	return repair_pset_check_backup(hint);
 }
 
 errno_t repair_fs_block_pack(aal_block_t *block, aal_stream_t *stream) {
