@@ -1,4 +1,4 @@
-/* Copyright 2001, 2002, 2003, 2004 by Hans Reiser, licensing governed by
+/* Copyright 2001-2005 by Hans Reiser, licensing governed by
    reiser4progs/COPYING.
    
    format40_repair.c -- repair methods for the default disk-layout plugin for 
@@ -11,7 +11,7 @@
 
 /* Update from the device only those fields which can be changed while 
    replaying. */
-errno_t format40_update(generic_entity_t *entity) {
+errno_t format40_update(reiser4_format_ent_t *entity) {
 	format40_t *format = (format40_t *)entity;
 	format40_super_t *super;
 	aal_block_t block;
@@ -46,7 +46,7 @@ errno_t format40_update(generic_entity_t *entity) {
 	return res;
 }
 
-errno_t format40_check_struct(generic_entity_t *entity, 
+errno_t format40_check_struct(reiser4_format_ent_t *entity, 
 			      backup_hint_t *hint, 
 			      format_hint_t *desc, 
 			      uint8_t mode)
@@ -75,7 +75,7 @@ errno_t format40_check_struct(generic_entity_t *entity,
 	if (aal_strncmp(super->sb_magic, FORMAT40_MAGIC, 
 			sizeof(FORMAT40_MAGIC))) 
 	{
-		fsck_mess("The on-disk format magic (%s) does not match the "
+		aal_error("The on-disk format magic (%s) does not match the "
 			  "format40 plugin one (%s).%s", super->sb_magic, 
 			  FORMAT40_MAGIC, mode == RM_BUILD ? " Fixed." : "");
 
@@ -88,6 +88,76 @@ errno_t format40_check_struct(generic_entity_t *entity,
 			res |= RE_FATAL;
 		}
 	}
+	
+	if (backup) {
+		/* Check the version if the version was backed up. */
+	        if ((get_sb_version(backup) > FORMAT40_VERSION) ||
+		    ((get_sb_version(super) > FORMAT40_VERSION) &&
+		     sb_update_backup(super))) {
+			aal_fatal("The on-disk format version (%u) is "
+				  "greater than the known version (%u). "
+				  "Please update reiser4progs and try again.",
+				  get_sb_version(super),FORMAT40_VERSION);
+			
+			return RE_FATAL;
+		}
+		
+		if ((get_sb_version(super) > get_sb_version(backup)) &&
+		    (get_sb_version(super) <= FORMAT40_VERSION) && 
+		    (sb_update_backup(super)))
+		{
+			/* Backup update is needed. */
+			aal_mess("Backup update is needed.%s", 
+				 mode == RM_CHECK ? "" : " Done.");
+
+			if (mode == RM_CHECK) {
+				res |= RE_FIXABLE;
+			} else {
+				set_sb_version(super, get_sb_version(super));
+				format40_mkdirty(format);
+			}
+		} else {
+			if (get_sb_version(super) != get_sb_version(backup)) {
+				fsck_mess("The on-disk format version (%u) does"
+					  " not match the backup one (%u).%s",
+					  get_sb_version(super), 
+					  get_sb_version(backup),
+					  mode == RM_BUILD ? " Fixed." : "");
+				
+				if (mode == RM_BUILD) {
+					set_sb_version(super, 
+						get_sb_version(backup));
+					
+					format40_mkdirty(format);
+				} else {
+					res |= RE_FATAL;
+				}
+			}
+			
+		}
+	} else {
+		if (get_sb_version(super) > FORMAT40_VERSION) {
+			int opt;
+
+			aal_mess("The on-disk format40 version (%u) is greater "
+				 "than the known version (%u). This probably "
+				 "means that reiser4progs is out of date. Fix "
+				 "the format40 version, only if you are sure "
+				 "this is a corruption.", get_sb_version(super),
+				 FORMAT40_VERSION);
+
+			if (mode != RM_BUILD)
+				return RE_FATAL;
+
+			opt = aal_yesno("Do you want to fix the format40 "
+					"version?");
+			if (opt != EXCEPTION_OPT_YES)
+				return -EINVAL;
+
+			set_sb_version(super, FORMAT40_VERSION);
+			format40_mkdirty(format);
+		}
+	}
 
 	/* Check the policy. */
 	if (desc->mask & (1 << PM_POLICY)) {
@@ -175,7 +245,7 @@ errno_t format40_check_struct(generic_entity_t *entity,
 		/* On-disk key is always correct as it is just 1 bit. */
 	}
 	
-	/* Check the block count. */
+	/* Check the block count, mkfs id and version number. */
 	if (backup) {
 		/* Check the block count. */
 		if (get_sb_block_count(super) != get_sb_block_count(backup)) {
@@ -194,6 +264,22 @@ errno_t format40_check_struct(generic_entity_t *entity,
 				res |= RE_FATAL;
 			}
 		}
+
+		/* Check the mkfs id. */
+		if (get_sb_mkfs_id(super) != get_sb_mkfs_id(backup)) {
+			fsck_mess("The on-disk format mkfs id (0x%x) does "
+				  "not match the backup one (0x%x).%s",
+				  get_sb_mkfs_id(super), get_sb_mkfs_id(backup),
+				  mode == RM_BUILD ? " Fixed." : "");
+
+			if (mode == RM_BUILD) {
+				set_sb_mkfs_id(super, get_sb_mkfs_id(backup));
+				format40_mkdirty(format);
+			} else {
+				res |= RE_FATAL;
+			}
+		}
+		
 	} else {
 		/* Check the block count. */
 		if (get_sb_block_count(super) != desc->blocks) {
@@ -209,29 +295,13 @@ errno_t format40_check_struct(generic_entity_t *entity,
 				res |= RE_FATAL;
 			}
 		}
-	}
-	
-	/* Check the mkfs id */
-	if (backup) {
-		if (get_sb_mkfs_id(super) != get_sb_mkfs_id(backup)) {
-			fsck_mess("The on-disk format mkfs id (0x%x) does "
-				  "not match the backup one (0x%x).%s",
-				  get_sb_mkfs_id(super), get_sb_mkfs_id(backup),
-				  mode == RM_BUILD ? " Fixed." : "");
 
-			if (mode == RM_BUILD) {
-				set_sb_mkfs_id(super, get_sb_mkfs_id(backup));
-				format40_mkdirty(format);
-			} else {
-				res |= RE_FATAL;
-			}
-		}
-	} 
+	}
 	
 	return res;
 }
 
-errno_t format40_pack(generic_entity_t *entity,
+errno_t format40_pack(reiser4_format_ent_t *entity,
 		      aal_stream_t *stream)
 {
 	rid_t pid;
@@ -244,7 +314,7 @@ errno_t format40_pack(generic_entity_t *entity,
 	format = (format40_t *)entity;
 
 	/* Write plugin id. */
-	pid = entity->plug->id.id;
+	pid = entity->plug->p.id.id;
 	aal_stream_write(stream, &pid, sizeof(pid));
 
 	/* Write data size. */
@@ -257,9 +327,9 @@ errno_t format40_pack(generic_entity_t *entity,
 	return 0;
 }
 
-generic_entity_t *format40_unpack(aal_device_t *device, 
-				  uint32_t blksize,
-				  aal_stream_t *stream)
+reiser4_format_ent_t *format40_unpack(aal_device_t *device, 
+				      uint32_t blksize,
+				      aal_stream_t *stream)
 {
 	uint32_t size;
 	format40_t *format;
@@ -289,7 +359,7 @@ generic_entity_t *format40_unpack(aal_device_t *device,
 		goto error_eostream;
 
 	format40_mkdirty(format);
-	return (generic_entity_t *)format;
+	return (reiser4_format_ent_t *)format;
 
  error_eostream:
 	aal_error("Can't unpack the disk format40. Stream is over?");
@@ -299,7 +369,7 @@ generic_entity_t *format40_unpack(aal_device_t *device,
 	return NULL;
 }
 
-void format40_print(generic_entity_t *entity,
+void format40_print(reiser4_format_ent_t *entity,
 		    aal_stream_t *stream,
 		    uint16_t options) 
 {
@@ -323,33 +393,35 @@ void format40_print(generic_entity_t *entity,
 	aal_stream_format(stream, "Format super block (%lu):\n",
 			  FORMAT40_BLOCKNR(format->blksize));
 	
-	aal_stream_format(stream, "plugin:\t\t%s\n",
-			  entity->plug->label);
+	aal_stream_format(stream, "plugin:\t\t%s\n", 
+			  entity->plug->p.label);
 	
-	aal_stream_format(stream, "description:\t%s\n",
-			  entity->plug->desc);
+	aal_stream_format(stream, "description:\t%s\n", 
+			  entity->plug->p.desc);
 
-	aal_stream_format(stream, "magic:\t\t%s\n",
+	aal_stream_format(stream, "version:\t%u\n", 
+			  get_sb_version(super));
+	
+	aal_stream_format(stream, "magic:\t\t%s\n", 
 			  super->sb_magic);
 	
-	aal_stream_format(stream, "flushes:\t%llu\n",
-			  get_sb_flushes(super));
-	
-	aal_stream_format(stream, "mkfs id:\t0x%x\n",
+	aal_stream_format(stream, "mkfs id:\t0x%x\n", 
 			  get_sb_mkfs_id(super));
     
-	aal_stream_format(stream, "blocks:\t\t%llu\n",
+	aal_stream_format(stream, "flushes:\t%llu\n", 
+			  get_sb_flushes(super));
+	
+	aal_stream_format(stream, "blocks:\t\t%llu\n", 
 			  get_sb_block_count(super));
 	
 	aal_stream_format(stream, "free blocks:\t%llu\n",
 			  get_sb_free_blocks(super));
 	
-	aal_stream_format(stream, "root block:\t%llu\n",
+	aal_stream_format(stream, "root block:\t%llu\n", 
 			  get_sb_root_block(super));
 
 	aal_stream_format(stream, "tail policy:\t0x%x (%s)\n",
-			  pid, plug ? plug->label:
-			  "absent");
+			  pid, plug ? plug->label: "absent");
 	
 	aal_stream_format(stream, "next oid:\t0x%llx\n",
 			  get_sb_oid(super));
@@ -397,15 +469,16 @@ errno_t format40_check_backup(backup_hint_t *hint) {
 		return RE_FATAL;
 	}
 	
-	/* Save the block count in the hint for the futher use. */
+	/* Save the block count & version in the hint for the futher use. */
 	hint->blocks = get_sb_block_count(backup);
+	hint->version = get_sb_version(backup);
 	
 	return 0;
 }
 
 /* Regenerate the format instance by the backup. */
-generic_entity_t *format40_regenerate(aal_device_t *device, 
-				      backup_hint_t *hint) 
+reiser4_format_ent_t *format40_regenerate(aal_device_t *device, 
+					  backup_hint_t *hint) 
 {
 	format40_backup_t *backup;
 	format40_super_t *super;
@@ -414,13 +487,22 @@ generic_entity_t *format40_regenerate(aal_device_t *device,
 	aal_assert("vpf-1737", hint != NULL);
 	aal_assert("vpf-1738", device != NULL);
 
+	backup = (format40_backup_t *)
+		(hint->block.data + hint->off[BK_FORMAT]);
+	
+	if (get_sb_version(backup) > FORMAT40_VERSION) {
+		fsck_mess("The reiser4 fs being repaired is formatted with "
+			  "format40 plugin version %u, whereas the used "
+			  "format40 plugin is of version %u. Please update "
+			  "reiser4progs and try again.", 
+			  get_sb_version(backup), FORMAT40_VERSION);
+		return NULL;
+	}
+	
 	/* Initializing format instance. */
 	if (!(format = aal_calloc(sizeof(*format), 0)))
 		return NULL;
 
-	backup = (format40_backup_t *)
-		(hint->block.data + hint->off[BK_FORMAT]);
-	
 	format->plug = &format40_plug;
 	format->device = device;
 	format->blksize = hint->block.size;
@@ -438,8 +520,10 @@ generic_entity_t *format40_regenerate(aal_device_t *device,
 	
 	set_sb_tree_height(super, 2);
 	set_sb_root_block(super, INVAL_BLK);
-
-	return (generic_entity_t *)format;
+	
+	set_sb_version(super, get_sb_version(backup));
+	
+	return (reiser4_format_ent_t *)format;
 }
 
 #endif
