@@ -1,4 +1,4 @@
-/* Copyright (C) 2001, 2002, 2003, 2004 by Hans Reiser, licensing governed by
+/* Copyright (C) 2001-2005 by Hans Reiser, licensing governed by
    reiser4progs/COPYING.
    
    rupasov_hash.c -- rupasov hash. */
@@ -49,25 +49,15 @@ uint64_t rupasov_hash_build(unsigned char *name, uint32_t len) {
 	return a;
 }
 
-static reiser4_hash_ops_t rupasov_hash_ops = {
+reiser4_hash_plug_t rupasov_hash_plug = {
+	.p = {
+		.id    = {HASH_RUPASOV_ID, 0, HASH_PLUG_TYPE},
+#ifndef ENABLE_MINIMAL
+		.label = "rupasov_hash",
+		.desc  = "Rupasov hash plugin.",
+#endif
+	},
+	
 	.build = rupasov_hash_build
 };
-
-static reiser4_plug_t rupasov_hash_plug = {
-	.cl    = class_init,
-	.id    = {HASH_RUPASOV_ID, 0, HASH_PLUG_TYPE},
-#ifndef ENABLE_MINIMAL
-	.label = "rupasov_hash",
-	.desc  = "Rupasov hash for reiser4. ",
-#endif
-	.o = {
-		.hash_ops = &rupasov_hash_ops
-	}
-};
-
-static reiser4_plug_t *rupasov_hash_start(reiser4_core_t *c) {
-	return &rupasov_hash_plug;
-}
-
-plug_register(rupasov_hash, rupasov_hash_start, NULL);
 #endif
