@@ -1,4 +1,4 @@
-/* Copyright (C) 2001, 2002, 2003, 2004 by Hans Reiser, licensing governed by
+/* Copyright (C) 2001-2005 by Hans Reiser, licensing governed by
    reiser4progs/COPYING.
    
    bbox40.c -- black box, reiser4 safe link items plugin implementation. */
@@ -28,7 +28,7 @@ static errno_t bbox40_prep_insert(reiser4_place_t *place,
 	hint->overhead = 0;
 
 	hint->count = 1;
-	hint->len = plug_call(link->key.plug->o.key_ops, bodysize);
+	hint->len = plugcall(link->key.plug, bodysize);
 	hint->len *= sizeof(uint64_t);
 
 	if (link->type == SL_TRUNCATE)
@@ -50,9 +50,7 @@ static errno_t bbox40_insert_units(reiser4_place_t *place,
 	
 	aal_assert("vpf-1573", link->key.plug != NULL);
 
-	size = plug_call(link->key.plug->o.key_ops, bodysize) * 
-		sizeof(uint64_t);
-	
+	size = plugcall(link->key.plug, bodysize) * sizeof(uint64_t);
 	aal_memcpy(place->body, &link->key.body, size);
 	
 	if (link->type == SL_TRUNCATE)
@@ -91,14 +89,12 @@ static errno_t bbox40_fetch_units(reiser4_place_t *place,
 
 	link = (slink_hint_t *)hint->specific;
 	
-	size = plug_call(place->key.plug->o.key_ops, bodysize) * 
-		sizeof(uint64_t);
-
+	size = plugcall(place->key.plug, bodysize) * sizeof(uint64_t);
 	link->key.plug = place->key.plug;
 	aal_memcpy(&link->key.body, place->body, size);
 	
 	/* FIXME: this is hardcoded, type should be obtained in another way. */
-	type = plug_call(place->key.plug->o.key_ops, get_offset, &place->key);
+	type = objcall(&place->key, get_offset);
 	
 	if (type == SL_TRUNCATE)
 		aal_memcpy(&link->size, place->body + size, sizeof(uint64_t));
@@ -117,7 +113,9 @@ static item_balance_ops_t balance_ops = {
 	.prep_shift	  = NULL,
 	.shift_units	  = NULL,
 	.collision	  = NULL,
+	.overhead	  = NULL,
 #endif
+	.init		  = NULL,
 	.lookup		  = NULL,
 	.fetch_key	  = NULL,
 	.maxposs_key	  = NULL,
@@ -128,7 +126,6 @@ static item_object_ops_t object_ops = {
 #ifndef ENABLE_MINIMAL
 	.size		  = NULL,
 	.bytes		  = NULL,
-	.overhead	  = NULL,
 	
 	.prep_write	  = NULL,
 	.write_units	  = NULL,
@@ -163,16 +160,15 @@ static item_debug_ops_t debug_ops = {
 };
 #endif
 
-static item_tree_ops_t tree_ops = {
-	.init		  = NULL,
-	.down_link	  = NULL,
+reiser4_item_plug_t bbox40_plug = {
+	.p = {
+		.id    = {ITEM_BLACKBOX40_ID, BLACK_BOX_ITEM, ITEM_PLUG_TYPE},
 #ifndef ENABLE_MINIMAL
-	.update_link	  = NULL
+		.label = "bbox40",
+		.desc  = "Safe link item plugin.",
 #endif
-};
+	},
 
-static reiser4_item_ops_t bbox40_ops = {
-	.tree		  = &tree_ops,
 	.object		  = &object_ops,
 	.balance	  = &balance_ops,
 #ifndef ENABLE_MINIMAL
@@ -180,22 +176,3 @@ static reiser4_item_ops_t bbox40_ops = {
 	.debug		  = &debug_ops
 #endif
 };
-
-static reiser4_plug_t bbox40_plug = {
-	.cl    = class_init,
-	.id    = {ITEM_BLACKBOX40_ID, BLACK_BOX_ITEM, ITEM_PLUG_TYPE},
-#ifndef ENABLE_MINIMAL
-	.label = "bbox40",
-	.desc  = "Safe link item plugin for reiser4. ",
-#endif
-	.o = {
-		.item_ops = &bbox40_ops
-	}
-};
-
-static reiser4_plug_t *bbox40_start(reiser4_core_t *c) {
-	bbox40_core = c;
-	return &bbox40_plug;
-}
-
-plug_register(bbox40, bbox40_start, NULL);
