@@ -1,4 +1,4 @@
-/* Copyright (C) 2001, 2002, 2003, 2004 by Hans Reiser, licensing governed by
+/* Copyright (C) 2001-2005 by Hans Reiser, licensing governed by
    reiser4progs/COPYING.
    
    cde40.c -- reiser4 directory entry plugin. */
@@ -9,7 +9,7 @@
 reiser4_core_t *cde40_core = NULL;
 
 inline uint32_t cde40_key_pol(reiser4_place_t *place) {
-	return plug_call(place->key.plug->o.key_ops, bodysize);
+	return plugcall(place->key.plug, bodysize);
 }
 
 /* Returns pointer to entry */
@@ -51,13 +51,12 @@ errno_t cde40_get_hash(reiser4_place_t *place, uint32_t pos,
 	hash = cde40_hash(place, pos);
 
 	/* Getting item key params */
-	locality = plug_call(place->key.plug->o.key_ops,
-			     get_locality, &place->key);
+	locality = objcall(&place->key, get_locality);
 
 	/* Building the full key from entry at @place */
-	plug_call(place->key.plug->o.key_ops, build_generic, key,
-		  KEY_FILENAME_TYPE, locality, ha_get_ordering(hash, pol),
-		  ha_get_objectid(hash, pol), ha_get_offset(hash, pol));
+	plugcall(place->key.plug, build_generic, key,
+		 KEY_FILENAME_TYPE, locality, ha_get_ordering(hash, pol),
+		 ha_get_objectid(hash, pol), ha_get_offset(hash, pol));
 
 	return 0;
 }
@@ -77,15 +76,10 @@ errno_t cde40_set_hash(reiser4_place_t *place, uint32_t pos,
 	pol = cde40_key_pol(place);
 	hash = cde40_hash(place, pos);
 	
-	ordering = plug_call(place->key.plug->o.key_ops,
-			     get_ordering, &place->key);
+	ordering = objcall(&place->key, get_ordering);
+	objectid = objcall(&place->key, get_fobjectid);
+	offset = objcall(&place->key, get_offset);
 	
-	objectid = plug_call(place->key.plug->o.key_ops,
-			     get_fobjectid, &place->key);
-	
-	offset = plug_call(place->key.plug->o.key_ops,
-			   get_offset, &place->key);
-
 	ha_set_ordering(hash, ordering, pol);
 	ha_set_objectid(hash, objectid, pol);
 	ha_set_offset(hash, offset, pol);
@@ -104,12 +98,12 @@ char *cde40_get_name(reiser4_place_t *place, uint32_t pos,
                                                                                         
         /* If name is long, we just copy it from the area after
            objectid. Otherwise we extract it from the entry hash. */
-        if (plug_call(key.plug->o.key_ops, hashed, &key)) {
+        if (objcall(&key, hashed)) {
                 char *ptr = (char *)(cde40_objid(place, pos) +
 				     ob_size(cde40_key_pol(place)));
                 aal_strncpy(buff, ptr, len);
         } else {
-		plug_call(key.plug->o.key_ops, get_name, &key, buff);
+		objcall(&key, get_name, buff);
         }
                                                                                         
         return buff;
@@ -134,7 +128,7 @@ static uint32_t cde40_get_len(reiser4_place_t *place, uint32_t pos) {
 	   Otherwise, entry name is stored in objectid and offset of the
 	   entry. This trick saves a lot of space in directories, because the
 	   average name is shorter than 15 symbols. */
-	if (plug_call(key.plug->o.key_ops, hashed, &key)) {
+	if (objcall(&key, hashed)) {
 		len += aal_strlen((char *)(cde40_objid(place, pos) +
 					   ob_size(pol))) + 1;
 	}
@@ -208,8 +202,7 @@ static int cde40_mergeable(reiser4_place_t *place1, reiser4_place_t *place2) {
 	aal_assert("umka-1582", place2 != NULL);
 
 	/* Items mergeable if their short keys match. */
-	return !plug_call(place1->key.plug->o.key_ops,
-			  compshort, &place1->key, &place2->key);
+	return !objcall(&place1->key, compshort, &place2->key);
 }
 
 /* Calculates the size of @count units (entries) in passed @place at passed
@@ -680,8 +673,7 @@ static errno_t cde40_prep_insert(reiser4_place_t *place, trans_hint_t *hint) {
 	aal_assert("umka-2424", place != NULL);
 	aal_assert("umka-2229", hint->count > 0);
 
-	pol = plug_call(hint->offset.plug->o.key_ops,
-			bodysize);
+	pol = plugcall(hint->offset.plug, bodysize);
 	
 	entry = (entry_hint_t *)hint->specific;
 	hint->len = (hint->count * en_size(pol));
@@ -691,9 +683,7 @@ static errno_t cde40_prep_insert(reiser4_place_t *place, trans_hint_t *hint) {
 
 		/* Calling key plugin for in odrer to find 
 		   out is passed name is long one or not. */
-		if (plug_call(hint->offset.plug->o.key_ops,
-			      hashed, &entry->offset))
-		{
+		if (objcall(&entry->offset, hashed)) {
 			/* Okay, name is long, so we need add 
 			   its length to estimated length. */
 			hint->len += aal_strlen(entry->name) + 1;
@@ -755,20 +745,17 @@ static int64_t cde40_insert_units(reiser4_place_t *place, trans_hint_t *hint) {
 		hash = &entry_hint->offset;
 		
 		/* Setting up ordering component of hash */
-		ord = plug_call(hash->plug->o.key_ops,
-				get_ordering, hash);
+		ord = objcall(hash, get_ordering);
 		
 		ha_set_ordering(entry, ord, pol);
 		
 		/* Setting up objectid component of hash */
-		oid = plug_call(hash->plug->o.key_ops,
-				get_fobjectid, hash);
+		oid = objcall(hash, get_fobjectid);
 		
 		ha_set_objectid(entry, oid, pol);
 
 		/* Setting up offset component of hash */
-		off = plug_call(hash->plug->o.key_ops,
-				get_offset, hash);
+		off = objcall(hash, get_offset);
 
 		ha_set_offset(entry, off, pol);
 
@@ -781,9 +768,7 @@ static int64_t cde40_insert_units(reiser4_place_t *place, trans_hint_t *hint) {
 		offset += ob_size(pol);
 
 		/* If key is long one we also count name length */
-		if (plug_call(place->key.plug->o.key_ops,
-			      hashed, &entry_hint->offset))
-		{
+		if (objcall(&entry_hint->offset, hashed)) {
 			uint32_t len = aal_strlen(entry_hint->name);
 
 			aal_memcpy(place->body + offset,
@@ -976,28 +961,20 @@ errno_t cde40_maxposs_key(reiser4_place_t *place,
 			  reiser4_key_t *key) 
 {
 	reiser4_key_t *maxkey;
-
+	
 	aal_assert("umka-1649", key != NULL);
 	aal_assert("umka-1648", place != NULL);
-
+	
 	aal_memcpy(key, &place->key, sizeof(*key));
-
+	
 	/* Getting maximal key from current key plugin. */
-	maxkey = plug_call(key->plug->o.key_ops, maximal);
-
+	maxkey = plugcall(key->plug, maximal);
+	
 	/* Setting up @key by mans of putting to it offset, ordering and
 	   objectid from values from maximal key. */
-    	plug_call(key->plug->o.key_ops, set_ordering,
-		  key, plug_call(key->plug->o.key_ops,
-				 get_ordering, maxkey));
-	
-    	plug_call(key->plug->o.key_ops, set_objectid,
-		  key, plug_call(key->plug->o.key_ops,
-				 get_objectid, maxkey));
-	
-	plug_call(key->plug->o.key_ops, set_offset,
-		  key, plug_call(key->plug->o.key_ops,
-				 get_offset, maxkey));
+    	plugcall(key->plug, set_ordering, key, objcall(maxkey, get_ordering));
+    	plugcall(key->plug, set_objectid, key, objcall(maxkey, get_objectid));
+	plugcall(key->plug, set_offset, key, objcall(maxkey, get_offset));
 	
 	return 0;
 }
@@ -1007,9 +984,7 @@ int cde40_comp_hash(reiser4_place_t *place, uint32_t pos, reiser4_key_t *key) {
 	reiser4_key_t curr;
 
 	cde40_get_hash(place, pos, &curr);
-
-	return plug_call(place->key.plug->o.key_ops,
-			 compfull, &curr, key);
+	return objcall(&curr, compfull, key);
 }
 
 /* Helper function that is used by lookup method for 
@@ -1070,7 +1045,9 @@ static item_balance_ops_t balance_ops = {
         .maxreal_key	  = cde40_maxreal_key,
 	.update_key	  = cde40_update_key,
 	.collision	  = cde40_collision,
+	.overhead	  = cde40_overhead,
 #endif
+	.init		  = NULL,
 	.units		  = cde40_units,
 	.lookup		  = cde40_lookup,
 	.fetch_key	  = cde40_fetch_key,
@@ -1087,7 +1064,6 @@ static item_object_ops_t object_ops = {
 	
 	.size		  = cde40_size,
 	.bytes		  = cde40_bytes,
-	.overhead	  = cde40_overhead,
 	 
 	.update_units	  = NULL,
 	.prep_write	  = NULL,
@@ -1115,39 +1091,20 @@ static item_repair_ops_t repair_ops = {
 };
 #endif
 
-static item_tree_ops_t tree_ops = {
-	.init		  = NULL,
-	.down_link	  = NULL,
+reiser4_item_plug_t cde40_plug = {
+	.p = {
+		.id    = {ITEM_CDE40_ID, DIR_ITEM, ITEM_PLUG_TYPE},
 #ifndef ENABLE_MINIMAL
-	.update_link	  = NULL
+		.label = "cde40",
+		.desc  = "Compound directory entry item plugin.",
 #endif
-};
-
-static reiser4_item_ops_t cde40_ops = {
-	.tree		  = &tree_ops,
+	},
+	
 	.object		  = &object_ops,
 	.balance	  = &balance_ops,
 #ifndef ENABLE_MINIMAL
 	.repair		  = &repair_ops,
 	.debug		  = &debug_ops
 #endif
+
 };
-
-static reiser4_plug_t cde40_plug = {
-	.cl    = class_init,
-	.id    = {ITEM_CDE40_ID, DIR_ITEM, ITEM_PLUG_TYPE},
-#ifndef ENABLE_MINIMAL
-	.label = "cde40",
-	.desc  = "Compound direntry for reiser4. ",
-#endif
-	.o = {
-		.item_ops = &cde40_ops
-	}
-};
-
-static reiser4_plug_t *cde40_start(reiser4_core_t *c) {
-	cde40_core = c;
-	return &cde40_plug;
-}
-
-plug_register(cde40, cde40_start, NULL);
