@@ -1,4 +1,4 @@
-/* Copyright (C) 2001, 2002, 2003, 2004 by Hans Reiser, licensing governed by
+/* Copyright (C) 2001-2005 by Hans Reiser, licensing governed by
    reiser4progs/COPYING.
    
    extent40.c -- reiser4 default extent plugin. */
@@ -10,7 +10,7 @@
 #include "extent40.h"
 #include "extent40_repair.h"
 
-reiser4_core_t *extent40_core;
+reiser4_core_t *extent40_core = NULL;
 
 /* Returns number of units in passed extent @place */
 uint32_t extent40_units(reiser4_place_t *place) {
@@ -206,8 +206,7 @@ static int64_t extent40_trunc_units(reiser4_place_t *place,
 
 	blksize = place_blksize(place);
 	extent = extent40_body(place) + place->pos.unit;
-	offset = plug_call(hint->offset.plug->o.key_ops, 
-			   get_offset, &hint->offset);
+	offset = objcall(&hint->offset, get_offset);
 
 	/* Get the amount of bytes that cannot be cut at the start. */
 	skip = (blksize - offset % blksize);
@@ -217,11 +216,10 @@ static int64_t extent40_trunc_units(reiser4_place_t *place,
 	/* Set the start key offset to the block border. */
 	offset += skip;
 	key = hint->offset;
-	plug_call(key.plug->o.key_ops, set_offset, &key, offset);
+	objcall(&key, set_offset, offset);
 
 	/* Get the amount of bytes from @offset through the end of the item. */
-	offset -= plug_call(hint->offset.plug->o.key_ops, 
-			   get_offset, &place->key);
+	offset -= objcall(&place->key, get_offset);
 	esize = extent40_size(place) - offset;
 
 	aal_assert("vpf-1759", offset % blksize == 0);
@@ -268,11 +266,8 @@ static int64_t extent40_trunc_units(reiser4_place_t *place,
 		for (i = 0; i < remove; i++) {
 			aal_hash_table_remove(hint->blocks, &key);
 
-			offset = plug_call(key.plug->o.key_ops,
-					   get_offset, &key);
-
-			plug_call(key.plug->o.key_ops, set_offset,
-				  &key, (offset + blksize));
+			offset = objcall(&key, get_offset);
+			objcall(&key, set_offset, (offset + blksize));
 		}
 		
 		start = et40_get_start(extent);
@@ -332,11 +327,10 @@ static int64_t extent40_trunc_units(reiser4_place_t *place,
 
 	/* Updating key if it makes sense. */
 	if (place->pos.unit == 0 && epos == 0 && count) {
-		offset = plug_call(place->key.plug->o.key_ops,
-				   get_offset, &place->key);
+		offset = objcall(&place->key, get_offset);
 		
-		plug_call(place->key.plug->o.key_ops, set_offset,
-			  &place->key, offset + count / blksize * blksize);
+		objcall(&place->key, set_offset, 
+			offset + count / blksize * blksize);
 	}
 	
 	return count + skip;
@@ -380,11 +374,9 @@ lookup_t extent40_lookup(reiser4_place_t *place,
 	extent = extent40_body(place);
 	units = extent40_units(place);
 
-	wanted = plug_call(hint->key->plug->o.key_ops,
-			   get_offset, hint->key);
+	wanted = objcall(hint->key, get_offset);
 
-	offset = plug_call(hint->key->plug->o.key_ops,
-			   get_offset, &place->key);
+	offset = objcall(&place->key, get_offset);
 	
 	for (i = 0; i < units; i++, extent++) {
 		offset += (et40_get_width(extent) * place_blksize(place));
@@ -406,6 +398,7 @@ static int64_t extent40_read_units(reiser4_place_t *place,
 {
 	void *buff;
 	uint32_t i;
+	errno_t res;
 	uint64_t read;
 	uint64_t count;
 	uint32_t blksize;
@@ -428,14 +421,15 @@ static int64_t extent40_read_units(reiser4_place_t *place,
 
 	extent40_fetch_key(place, &key);
 	blksize = place_blksize(place);
+	read = count;
 
 	/* Initializing read offset */
-	read_offset = plug_call(hint->offset.plug->o.key_ops,
-				get_offset, &hint->offset);
+	read_offset = objcall(&hint->offset, get_offset);
+	rel_offset = read_offset - objcall(&key, get_offset);
 
 	/* Loop through the units until needed amount of data is read or extent
 	   item is over. */
-	for (read = count, i = place->pos.unit;
+	for (i = place->pos.unit;
 	     i < extent40_units(place) && count > 0; i++)
 	{
 		uint32_t size;
@@ -446,7 +440,6 @@ static int64_t extent40_read_units(reiser4_place_t *place,
 
 		/* Calculating start block for read. */
 		start = et40_get_start(extent + i);
-		rel_offset = read_offset - extent40_offset(place, i);
 		blk = start + (rel_offset / blksize);
 
 		if (start == EXTENT_HOLE_UNIT) {
@@ -467,68 +460,70 @@ static int64_t extent40_read_units(reiser4_place_t *place,
 
 				aal_memset(buff, 0, size);
 			}
-		} else {
+		} else while (blk < start + et40_get_width(extent + i) &&
+			      count > 0)
+		{
 			/* Loop though the one unit. */
-			while (blk < start + et40_get_width(extent + i) &&
-			       count > 0)
-			{
-				uint32_t rest;
+			uint32_t rest;
 
-				rest = blksize - (read_offset % blksize);
-				
-				if ((size = count) > rest)
-					size = rest;
-				
-				/* Initilaizing offset of block needed data lie
-				   in. It is needed for getting block from data
-				   cache. */
-				block_offset = read_offset - (read_offset &
-							      (blksize - 1));
+			rest = blksize - (read_offset % blksize);
 
-				plug_call(key.plug->o.key_ops, set_offset,
-					  &key, block_offset);
+			if ((size = count) > rest)
+				size = rest;
 
-				/* Getting block from the cache. */
-				if (!(block = aal_hash_table_lookup(hint->blocks, &key))) {
-					reiser4_key_t *ins_key;
-					
-					/* If block is not found in cache, we
-					   read it and put to cache. */
-					aal_device_t *device = extent40_device(place);
-				
-					if (!(block = aal_block_load(device, blksize,
-								     blk)))
-					{
-						return -EIO;
-					}
+			/* Initilaizing offset of block needed data lie
+			   in. It is needed for getting block from data
+			   cache. */
+			block_offset = read_offset - (read_offset &
+						      (blksize - 1));
 
-					if (!(ins_key = aal_calloc(sizeof(*ins_key), 0)))
-						return -ENOMEM;
+			objcall(&key, set_offset, block_offset);
 
-					aal_memcpy(ins_key, &key, sizeof(key));
-					
-					aal_hash_table_insert(hint->blocks, ins_key, block);
-				}
+			/* Getting block from the cache. */
+			block = aal_hash_table_lookup(hint->blocks, &key);
+			if (!block) {
+				reiser4_key_t *ins_key;
 
-				/* Copying data from found (loaded) block to
-				   @buff. */
-				aal_memcpy(buff, block->data +
-					   (read_offset % blksize), size);
+				/* If block is not found in cache, we
+				   read it and put to cache. */
+				aal_device_t *device = extent40_device(place);
 
-				buff += size;
-				count -= size;
+				block = aal_block_load(device, blksize, blk);
+				if (!block)
+					return -EIO;
 
-				/* Updating read offset and blk next read will
-				   be performed from. */
-				read_offset += size;
+				ins_key = aal_calloc(sizeof(*ins_key), 0);
+				if (!ins_key)
+					return -ENOMEM;
 
-				if ((read_offset % blksize) == 0)
-					blk++;
+				aal_memcpy(ins_key, &key, sizeof(key));
+				aal_hash_table_insert(hint->blocks,
+						      ins_key, block);
 			}
+
+			/* Copying data from found (loaded) block to
+			   @buff. */
+			aal_memcpy(buff, block->data +
+				   (read_offset % blksize), size);
+
+			buff += size;
+			count -= size;
+
+			/* Updating read offset and blk next read will
+			   be performed from. */
+			read_offset += size;
+
+			if ((read_offset % blksize) == 0)
+				blk++;
 		}
+
+		res = extent40_core->tree_ops.mpressure(place->node->tree);
+		if (res) return res;
+		
+		rel_offset = 0;
 	}
 	
-	return read;
+	return read - count;
 }
 #else
 /* Reads @count bytes of extent data from the extent item at passed @pos into
@@ -560,12 +555,11 @@ static int64_t extent40_read_units(reiser4_place_t *place, trans_hint_t *hint) {
 	blksize = place_blksize(place);
 	secsize = extent40_secsize(place);
 
-	read_offset = plug_call(hint->offset.plug->o.key_ops,
-				get_offset, &hint->offset);
+	read_offset = objcall(&hint->offset, get_offset);
+	rel_offset = read_offset - objcall(&key, get_offset);
 	
-	rel_offset = read_offset - extent40_offset(place, place->pos.unit);
-	
-	for (read = count, i = place->pos.unit;
+	read = count;
+	for (i = place->pos.unit;
 	     i < extent40_units(place) && count > 0; i++, rel_offset = 0)
 	{
 		uint32_t blkchunk;
@@ -629,9 +623,11 @@ static int64_t extent40_read_units(reiser4_place_t *place, trans_hint_t *hint) {
 			if (blklocal % blksize == 0)
 				blk++;
 		}
+
+		rel_offset = 0;
 	}
 	
-	return (uint64_t)read;
+	return (uint64_t)read - count;
 }
 #endif
 
@@ -811,7 +807,6 @@ static int64_t extent40_alloc_block(reiser4_place_t *place,
 
 	/* Prepare the key of the new allocated block. */
 	aal_memcpy(&key, &place->key, sizeof(key));
-	plug_call(key.plug->o.key_ops, set_offset, &key, offset);
 
 	bytes = count;
 	
@@ -822,6 +817,9 @@ static int64_t extent40_alloc_block(reiser4_place_t *place,
 	}
 	
 	for (; count > 0; count -= blksize, offset += blksize) {
+		/* Update @key offset. */
+		objcall(&key, set_offset, offset);
+		
 		/* Calculating size to be written this time. */
 		if (!(block = aal_block_alloc(device, blksize, 0)))
 			return -ENOMEM;
@@ -833,10 +831,8 @@ static int64_t extent40_alloc_block(reiser4_place_t *place,
 
 		aal_hash_table_insert(blocks, ins_key, block);
 
-		/* Update @key offset. */
-		plug_call(key.plug->o.key_ops, set_offset, &key, offset);
 	}
-
+		
 	return bytes;
 }
 
@@ -922,23 +918,20 @@ static errno_t extent40_prep_write(reiser4_place_t *place, trans_hint_t *hint) {
 		/* Inserting a new item or appending units. */
 
 		/* Set maxkey as the limit of the operation. */
-		max_off = plug_call(hint->maxkey.plug->o.key_ops,
-				    get_offset, &hint->maxkey);
-
+		max_off = objcall(&hint->maxkey, get_offset);
 		max_off += hint->count;
-
-		plug_call(hint->maxkey.plug->o.key_ops,
-			  set_offset, &hint->maxkey, max_off);
+		objcall(&hint->maxkey, set_offset, max_off);
 
 		if (place->pos.unit != MAX_UINT32) {
-			uint64_t start;
+			uint64_t width;
 
 			/* If new data could be attached to the end of existent,
 			   no new unit is needed. It is possible if we write data 
 			   over not allocated unit or hole over the hole. */
 			extent = extent40_body(place) + units - 1;
 			start = et40_get_start(extent);
-
+			width = et40_get_width(extent);
+			
 			if ((start == EXTENT_UNALLOC_UNIT && hint->specific) ||
 			    (start == EXTENT_HOLE_UNIT && !hint->specific))
 			{
@@ -960,12 +953,9 @@ static errno_t extent40_prep_write(reiser4_place_t *place, trans_hint_t *hint) {
 
 		/* Getting maximal real key, it limits the amount of 
 		   data being written at once. */
-		uni_off = plug_call(key.plug->o.key_ops,
-				    get_offset, &key);
-
-		ins_off = plug_call(hint->offset.plug->o.key_ops,
-				    get_offset, &hint->offset);
-
+		uni_off = objcall(&key, get_offset);
+		ins_off = objcall(&hint->offset, get_offset);
+		
 		aal_assert("vpf-1697", uni_off <= ins_off);
 
 		extent = extent40_body(place) + unit_pos;
@@ -1016,9 +1006,7 @@ static errno_t extent40_prep_write(reiser4_place_t *place, trans_hint_t *hint) {
 			}
 		}
 
-		plug_call(hint->maxkey.plug->o.key_ops,
-			  set_offset, &hint->maxkey, ins_off);
-
+		objcall(&hint->maxkey, set_offset, ins_off);
 		hint->insert_flags |= ET40_OVERWRITE;
 	}
 
@@ -1048,6 +1036,7 @@ static int64_t extent40_write_units(reiser4_place_t *place, trans_hint_t *hint)
 	uint32_t width;
 	uint32_t units;
 
+	errno_t res;
 	char *buff;
 
 
@@ -1068,10 +1057,8 @@ static int64_t extent40_write_units(reiser4_place_t *place, trans_hint_t *hint)
 	blksize = place_blksize(place);
 
 	/* Get the offset range being handled: ins_off, max_off. */
-	ins_off = plug_call(hint->offset.plug->o.key_ops,
-			    get_offset, &hint->offset);
-	max_off = plug_call(hint->maxkey.plug->o.key_ops,
-			    get_offset, &hint->maxkey);
+	ins_off = objcall(&hint->offset, get_offset);
+	max_off = objcall(&hint->maxkey, get_offset);
 
 	total = max_off - ins_off;
 	count = total;
@@ -1217,10 +1204,8 @@ static int64_t extent40_write_units(reiser4_place_t *place, trans_hint_t *hint)
 	extent40_fetch_key(place, &key);
 
 	extent = extent40_body(place) + place->pos.unit;
-	uni_off = plug_call(key.plug->o.key_ops, get_offset, &key);
-	
-	ins_off = plug_call(hint->offset.plug->o.key_ops,
-			    get_offset, &hint->offset);
+	uni_off = objcall(&key, get_offset);
+	ins_off = objcall(&hint->offset, get_offset);
 	
 	count = total;
 
@@ -1237,7 +1222,7 @@ static int64_t extent40_write_units(reiser4_place_t *place, trans_hint_t *hint)
 			size = count;
 
 		/* Preparing key for getting data by it. */
-		plug_call(key.plug->o.key_ops, set_offset, &key, blk_off);
+		objcall(&key, set_offset, blk_off);
 
 		/* Checking if we write data inside item. */
 		aal_assert("vpf-1705", blk_off + size <= max_off);
@@ -1277,6 +1262,9 @@ static int64_t extent40_write_units(reiser4_place_t *place, trans_hint_t *hint)
 		}
 	}
 	
+	if ((res = extent40_core->tree_ops.mpressure(place->node->tree)))
+		return res;
+	
 	place_mkdirty(place);
 	return total;
 }
@@ -1479,7 +1467,9 @@ static item_balance_ops_t balance_ops = {
 	.shift_units	  = extent40_shift_units,
 	.maxreal_key	  = extent40_maxreal_key,
 	.collision	  = NULL,
+	.overhead	  = NULL,
 #endif
+	.init		  = NULL,
 	.units		  = extent40_units,
 	.lookup		  = extent40_lookup,
 	.fetch_key	  = extent40_fetch_key,
@@ -1498,7 +1488,6 @@ static item_object_ops_t object_ops = {
 	.layout		  = extent40_layout,
 	.size		  = extent40_size,
 	.bytes		  = extent40_bytes,
-	.overhead	  = NULL,
 #endif
 	.read_units	  = extent40_read_units,
 	.fetch_units	  = extent40_fetch_units
@@ -1521,16 +1510,15 @@ static item_debug_ops_t debug_ops = {
 };
 #endif
 
-static item_tree_ops_t tree_ops = {
-	.init		  = NULL,
-	.down_link	  = NULL,
+reiser4_item_plug_t extent40_plug = {
+	.p = {
+		.id	= {ITEM_EXTENT40_ID, EXTENT_ITEM, ITEM_PLUG_TYPE},
 #ifndef ENABLE_MINIMAL
-	.update_link	  = NULL,
+		.label = "extent40",
+		.desc  = "Extent file body item plugin.",
 #endif
-};
-
-static reiser4_item_ops_t extent40_ops = {
-	.tree		  = &tree_ops,
+	},
+	
 	.object		  = &object_ops,
 	.balance	  = &balance_ops,
 #ifndef ENABLE_MINIMAL
@@ -1538,22 +1526,3 @@ static reiser4_item_ops_t extent40_ops = {
 	.debug		  = &debug_ops,
 #endif
 };
-
-static reiser4_plug_t extent40_plug = {
-	.cl    = class_init,
-	.id    = {ITEM_EXTENT40_ID, EXTENT_ITEM, ITEM_PLUG_TYPE},
-#ifndef ENABLE_MINIMAL
-	.label = "extent40",
-	.desc  = "Extent item for reiser4. ",
-#endif
-	.o = {
-		.item_ops = &extent40_ops
-	}
-};
-
-static reiser4_plug_t *extent40_start(reiser4_core_t *c) {
-	extent40_core = c;
-	return &extent40_plug;
-}
-
-plug_register(extent40, extent40_start, NULL);
