@@ -1,4 +1,4 @@
-/* Copyright (C) 2001, 2002, 2003, 2004 by Hans Reiser, licensing governed by
+/* Copyright (C) 2001-2005 by Hans Reiser, licensing governed by
    reiser4progs/COPYING.
    
    node40_repair.h -- reiser4 node plugin repair functions. */
@@ -19,7 +19,7 @@ extern errno_t node40_pack(reiser4_node_t *entity,
 			   aal_stream_t *stream);
 
 extern reiser4_node_t *node40_unpack(aal_block_t *block,
-				    reiser4_plug_t *kplug,
+				    reiser4_key_plug_t *kplug,
 				    aal_stream_t *stream);
 
 extern void node40_print(reiser4_node_t *entity, aal_stream_t *stream,
