@@ -1,4 +1,4 @@
-/* Copyright (C) 2001, 2002, 2003, 2004 by Hans Reiser, licensing governed by
+/* Copyright (C) 2001-2005 by Hans Reiser, licensing governed by
    reiser4progs/COPYING.
    
    oid40_repair.h -- reiser4 oid allocator plugin repair functions. */
@@ -9,7 +9,7 @@
 #ifndef ENABLE_MINIMAL
 #include <reiser4/plugin.h>
 
-extern void oid40_print(generic_entity_t *entity,
+extern void oid40_print(reiser4_oid_ent_t *entity,
 			aal_stream_t *stream,
 			uint16_t options);
 
