@@ -1,4 +1,4 @@
-/* Copyright 2001, 2002, 2003, 2004 by Hans Reiser, licensing governed by
+/* Copyright 2001-2005 by Hans Reiser, licensing governed by
    reiser4progs/COPYING.
    
    sdext_flags_repair.c -- inode flags stat data extension plugin recovery
@@ -12,14 +12,14 @@
 
 errno_t sdext_flags_check_struct(stat_entity_t *stat, repair_hint_t *hint) {
 	aal_assert("umka-3081", stat != NULL);
-	aal_assert("umka-3082", stat->ext_plug != NULL);
+	aal_assert("umka-3082", stat->plug != NULL);
 	
 	if (stat->offset + sizeof(sdext_flags_t) > stat->place->len) {
 		fsck_mess("Node (%llu), item (%u), [%s]: does not look "
 			  "like a valid (%s) statdata extension.", 
 			  place_blknr(stat->place), stat->place->pos.item,
 			  print_key(sdext_flags_core, &stat->place->key),
-			  stat->ext_plug->label);
+			  stat->plug->p.label);
 		return RE_FATAL;
 	}
 	
