@@ -1,4 +1,4 @@
-/* Copyright (C) 2001, 2002, 2003, 2004 by Hans Reiser, licensing governed by
+/* Copyright (C) 2001-2005 by Hans Reiser, licensing governed by
    reiser4progs/COPYING.
    
    sdext_unix.h -- stat data exception plugin, that implements unix stat data
@@ -11,21 +11,17 @@
 #include <reiser4/plugin.h>
 
 /* stat-data extension for files with non-standard plugin. */
-struct sdext_plug_slot {
+typedef struct sdext_plug_slot {
         d16_t member;
         d16_t plug;
-} __attribute__((packed));
+} __attribute__((packed)) sdext_plug_slot_t;
 
-typedef struct sdext_plug_slot sdext_plug_slot_t;
-
-struct sdext_plug {
+typedef struct sdext_plug {
         d16_t count;
         sdext_plug_slot_t slot[0];
-}  __attribute__((packed));
+}  __attribute__((packed)) sdext_plug_t;
 
-typedef struct sdext_plug sdext_plug_t;
-
-extern reiser4_core_t *sdext_plug_core;
+extern reiser4_core_t *sdext_pset_core;
 
 #define sdext_plug_get_count(ext)		aal_get_le16(ext, count)
 #define sdext_plug_set_count(ext, val)		aal_set_le16(ext, count, (val))
