@@ -1,4 +1,4 @@
-/* Copyright (C) 2001, 2002, 2003, 2004 by Hans Reiser, licensing governed by
+/* Copyright (C) 2001-2005 by Hans Reiser, licensing governed by
    reiser4progs/COPYING.
    
    print.c -- filesystem print related code. */
@@ -144,7 +144,7 @@ void debugfs_print_status(reiser4_fs_t *fs) {
 void debugfs_print_format(reiser4_fs_t *fs) {
 	aal_stream_t stream;
 
-	if (!fs->format->ent->plug->o.format_ops->print) {
+	if (!fs->format->ent->plug->print) {
 		aal_info("Format print method is not "
 			 "implemented.");
 		return;
@@ -161,7 +161,7 @@ void debugfs_print_format(reiser4_fs_t *fs) {
 void debugfs_print_oid(reiser4_fs_t *fs) {
 	aal_stream_t stream;
     
-	if (!fs->oid->ent->plug->o.oid_ops->print) {
+	if (!fs->oid->ent->plug->print) {
 		aal_info("Oid allocator print method is "
 			 "not implemented.");
 		return;
@@ -199,13 +199,11 @@ void debugfs_print_journal(reiser4_fs_t *fs) {
 	aal_stream_fini(&stream);
 }
 
-struct fprint_hint {
+typedef struct fprint_hint {
 	blk_t old;
 	void *data;
 	uint32_t flags;
-};
-
-typedef struct fprint_hint fprint_hint_t;
+} fprint_hint_t;
 
 /* Prints item at passed @place */
 static errno_t fprint_process_place(
@@ -257,7 +255,7 @@ errno_t debugfs_print_file(
 		if ((res = reiser4_object_metadata(object, place_func, &hint)))
 		{
 			aal_error("Can't print object %s metadata.", 
-				  reiser4_print_inode(&object->ent->object));
+				  reiser4_print_inode(&object->info.object));
 		}
 	}
 
